// A program to display an extruded surface
//
// Use 'l' to render the orignal line strip
// Use 'd' to render the orignal line strip and the extruded libe strip
// Use 's' to render the extruded surface 

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include "Trackball.h"
#include "Lighting.h"
#include "Geometry.h"

const int windowWidth=400;
const int windowHeight=400;

typedef enum {SINGLE_LINE, EXTRUDED_LINE, EXTRUDED_SURFACE} RENDER_OPTION;
RENDER_OPTION current_render_option=EXTRUDED_SURFACE;

// material properties
GLfloat mat_specular[4];
GLfloat mat_ambient_and_diffuse[4];
GLfloat mat_shininess[1];

CTrackball trackball;
CLighting lighting;

// the original 2D line 
const int numVertices=10;
const float vertices[numVertices][2] =  {{0,1},{0.25f,0},{0.5f,0.6f},{0.75f,0},{1,1},
    						{0.9f,1},{0.75f,0.4f},{0.5f,1},{0.25f,0.4f},{0.1f,1}};


void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'l': current_render_option=SINGLE_LINE; 
					glutPostRedisplay(); break;
		case 'd': current_render_option=EXTRUDED_LINE; 
					glutPostRedisplay(); break;
		case 's': current_render_option=EXTRUDED_SURFACE; 
					glutPostRedisplay(); break;
		default: trackball.tbKeyboard(key);
	}
}

void display(void)
{
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
    gluLookAt(0,0,6, 0,0,0, 0,1,0); // camera is on the z-axis
	trackball.tbMatrix();			// rotate scene using the trackball

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	int i;
	switch (current_render_option)
	{
	case SINGLE_LINE:
		glLineWidth(1.0);
		glBegin(GL_LINE_STRIP);
		for(i=0;i<numVertices;i++)
			glVertex3f(vertices[i][0], vertices[i][1], 0);
		glVertex3f(vertices[0][0], vertices[0][1], 0);
		glEnd();
		break;
	case EXTRUDED_LINE:
		glLineWidth(2.0);
		glBegin(GL_LINE_STRIP);
		for(i=0;i<numVertices;i++)
			glVertex3f(vertices[i][0], vertices[i][1], 0);
		glVertex3f(vertices[0][0], vertices[0][1], 0);
		glEnd();
		glLineWidth(1.0);
		glBegin(GL_LINE_STRIP);
		for(i=0;i<numVertices;i++)
			glVertex3f(vertices[i][0], vertices[i][1], -0.4);
		glVertex3f(vertices[0][0], vertices[0][1], -0.4);
		glEnd();
		break;
	case EXTRUDED_SURFACE:
	glEnable(GL_CULL_FACE);	// for efficiency cull back faces of solid extruded object
	glBegin(GL_QUAD_STRIP);
	for(i=0;i<=numVertices;i++)
	{
		glVertex3f(vertices[i%10][0], vertices[i%10][1], 0.0);
		glVertex3f(vertices[i%10][0], vertices[i%10][1], -0.4);
		CVec3df v1(0,0,-0.4);	
		CVec3df v2( vertices[(i+1)%10][0]-vertices[i%10][0],
					vertices[(i+1)%10][1]-vertices[i%10][1],0);
		CVec3df n=cross(v1,v2); 
		n.normaliseDestructive();
		glNormal3fv(n.getArray());
	}
	glEnd();
	// Front face
	glBegin(GL_QUAD_STRIP);
	glNormal3f(0,0,1.0);
	glVertex3f(vertices[4][0], vertices[4][1], 0.0);
	glVertex3f(vertices[5][0], vertices[5][1], 0.0);
	glVertex3f(vertices[3][0], vertices[3][1], 0.0);
	glVertex3f(vertices[6][0], vertices[6][1], 0.0);
	glVertex3f(vertices[2][0], vertices[2][1], 0.0);
	glVertex3f(vertices[7][0], vertices[7][1], 0.0);
	glVertex3f(vertices[1][0], vertices[1][1], 0.0);
	glVertex3f(vertices[8][0], vertices[8][1], 0.0);
	glVertex3f(vertices[0][0], vertices[0][1], 0.0);
	glVertex3f(vertices[9][0], vertices[9][1], 0.0);
	glEnd();
	// Back face
	glBegin(GL_QUAD_STRIP);
	glNormal3f(0,0,-1.0);
	glVertex3f(vertices[0][0], vertices[0][1], -0.4);
	glVertex3f(vertices[9][0], vertices[9][1], -0.4);
	glVertex3f(vertices[1][0], vertices[1][1], -0.4);
	glVertex3f(vertices[8][0], vertices[8][1], -0.4);
	glVertex3f(vertices[2][0], vertices[2][1], -0.4);
	glVertex3f(vertices[7][0], vertices[7][1], -0.4);
	glVertex3f(vertices[3][0], vertices[3][1], -0.4);
	glVertex3f(vertices[6][0], vertices[6][1], -0.4);
	glVertex3f(vertices[4][0], vertices[4][1], -0.4);
	glVertex3f(vertices[5][0], vertices[5][1], -0.4);
	glEnd();
	}	
	glDisable(GL_CULL_FACE);
	glFlush ();
	glutSwapBuffers();
}

void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1,1,1,1);	// RGB-value for black
	// enable depth buffering
	glEnable(GL_DEPTH_TEST);
	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(33,1,2,8);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	// material properties 
	mat_ambient_and_diffuse[0]=0.8;		// red material ...
	mat_ambient_and_diffuse[1]=0.0;
	mat_ambient_and_diffuse[2]=0.0;
	mat_ambient_and_diffuse[3]=1;
	mat_specular[0]=0.8f;				// ... with white highlights
	mat_specular[1]=0.8f;				// if light source is reflected
	mat_specular[2]=0.8f;				// on the material surface.
	mat_specular[3]=1;
	mat_shininess[0]=100;
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_ambient_and_diffuse);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	// enable shading and lighting
	lighting.enable();
	glShadeModel(GL_FLAT);
}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a double buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Extruded Surface");
	init ();								// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
